@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import weakref
 from collections import defaultdict
 from typing import Any
 from typing import Dict
@@ -15,6 +16,16 @@ from geneticengine.core.utils import is_abstract
 from geneticengine.core.utils import is_terminal
 
 
+class GengyList(list):
+    def __init__(self, typ, vals):
+        super().__init__(vals)
+        self.typ = typ
+        self.gengy_init_values = vals
+
+    def new_like(self, *newargs):
+        return GengyList(self.typ, newargs)
+
+
 def relabel_nodes(i: TreeNode, g: Grammar) -> tuple[int, int, dict[type, list[Any]]]:
     non_terminals = g.non_terminals
     children: list[Any]
@@ -24,7 +35,7 @@ def relabel_nodes(i: TreeNode, g: Grammar) -> tuple[int, int, dict[type, list[An
     distance_to_term = 1
     types_this_way = defaultdict(lambda: [])
     types_this_way[type(i)] = [i]
-    if is_terminal(type(i), non_terminals):
+    if is_terminal(type(i), non_terminals) and (not isinstance(i, list)):
         if not is_builtin(type(i)):
             i.gengy_labeled = True
             i.gengy_distance_to_term = 1
@@ -57,11 +68,10 @@ def relabel_nodes(i: TreeNode, g: Grammar) -> tuple[int, int, dict[type, list[An
             for (k, v) in thisway.items():
                 types_this_way[k].extend(v)
 
-    if not isinstance(i, list):
-        i.gengy_labeled = True
-        i.gengy_distance_to_term = distance_to_term
-        i.gengy_nodes = number_of_nodes
-        i.gengy_types_this_way = types_this_way
+    i.gengy_labeled = True
+    i.gengy_distance_to_term = distance_to_term
+    i.gengy_nodes = number_of_nodes
+    i.gengy_types_this_way = types_this_way
     return number_of_nodes, distance_to_term, types_this_way
 
 
